from pinpong.board import Board
from tkinter import Tk, Label
from PIL import ImageTk, Image
from tkintermapview import TkinterMapView
from lib.DFRobot_GNSS_I2C import DFRobot_GNSS_I2C, MODE_GPS_BEIDOU_GLONASS


SCREEN_WIDTH: int = 240
SCREEN_HEIGHT: int = 320

MINIMUM_SATELLITES: int = 3
DELAY_MILLISECONDS: int = 2000

TILE_SERVER: str = 'https://a.tile.openstreetmap.org/{z}/{x}/{y}.png'
TILE_ZOOM: int = 15


def create_map(display, latitude, longitude) -> None:
    """
    Create a map with marker
    :param display: tkinter display object
    :param latitude: float value of latitude
    :param longitude: float value of longitude
    :return: None
    """
    gmap = TkinterMapView(display, width=SCREEN_WIDTH, height=SCREEN_HEIGHT)
    gmap.pack(fill='both', expand=True)
    gmap.set_tile_server(TILE_SERVER)
    gmap.set_position(latitude[0], longitude[0], marker=True)
    gmap.set_zoom(TILE_ZOOM)


def check_satellite() -> None:
    """
    Check if minimum of satellites is reached
    :return: None
    """
    global satellite_found
    global screen
    global sensor
    global image_label

    num_satellites = sensor.get_num_sta_used()
    current_time = sensor.get_time()
    current_date = sensor.get_date()

    print(f"Found {num_satellites} satellites at {current_date} {current_time}")

    if num_satellites > MINIMUM_SATELLITES and not satellite_found:
        satellite_found = True

        lat = sensor.get_lat()
        long = sensor.get_lon()

        sensor.set_disable_power()

        image_label.pack_forget()

        create_map(display=screen, latitude=lat, longitude=long)

    if not satellite_found:
        screen.after(DELAY_MILLISECONDS, check_satellite)


if __name__ == '__main__':
    Board().begin()

    sensor = DFRobot_GNSS_I2C()
    sensor.set_gnss_mode(MODE_GPS_BEIDOU_GLONASS)
    sensor.set_enable_power()
    sensor.set_rgb_on()

    satellite_found = False

    screen = Tk()
    screen.geometry(f'{SCREEN_WIDTH}x{SCREEN_HEIGHT}+0+0')
    screen.resizable(False, False)

    image_label = Label(screen)
    image_label.pack(fill='both')
    image_src = Image.open("img/satellite.png")
    image = ImageTk.PhotoImage(image_src)
    image_label.config(image=image)

    check_satellite()

    screen.mainloop()
